package repositories

import (
	"context"
	"database/sql"

	"github.com/dmitriygnatenko/internal/interfaces"
)

type articleTagRepository struct {
	db *sql.DB
}

func InitArticleTagRepository(db *sql.DB) interfaces.IArticleTagRepository {
	return articleTagRepository{db: db}
}

func (a articleTagRepository) Add(ctx context.Context, articleID int, tagIDs []int) error {
	if len(tagIDs) == 0 {
		return nil
	}

	query := "INSERT INTO " + articleTagTableName + " (article_id, tag_id) VALUES "

	var vals []interface{}
	for _, tagID := range tagIDs {
		query += "(?, ?),"
		vals = append(vals, articleID, tagID)
	}
	query = query[0 : len(query)-1]

	_, err := a.db.ExecContext(ctx, query, vals...)

	return err
}

func (a articleTagRepository) Delete(ctx context.Context, articleID int, tagIDs []int) error {
	if len(tagIDs) == 0 {
		return nil
	}

	query := "DELETE FROM " + articleTagTableName + " WHERE article_id = ? AND tag_id IN ("

	var vals []interface{}
	vals = append(vals, articleID)
	for _, tagID := range tagIDs {
		query += "?,"
		vals = append(vals, tagID)
	}
	query = query[0:len(query)-1] + ")"

	_, err := a.db.ExecContext(ctx, query, vals...)

	return err
}

func (a articleTagRepository) DeleteByArticleID(ctx context.Context, articleID int) error {
	query := "DELETE FROM " + articleTagTableName + " WHERE article_id = ?"

	_, err := a.db.ExecContext(ctx, query, articleID)

	return err
}
