package handler

import (
	"github.com/dmitriygnatenko/internal/interfaces"
	"github.com/dmitriygnatenko/internal/mapper"
	"github.com/gofiber/fiber/v2"
)

const allPreviewArticlesCacheKey = "all-preview-articles"

func MainPageHandler(sp interfaces.IServiceProvider) fiber.Handler {
	return func(ctx *fiber.Ctx) error {
		renderData, found := sp.GetCacheService().Get(allPreviewArticlesCacheKey)

		if !found {
			articles, err := sp.GetArticleRepository().GetAllPreview(ctx.Context())
			if err != nil {
				return err
			}

			articlesDTO, err := mapper.ConvertArticlePreviewModelsToDTO(articles)
			if err != nil {
				return err
			}

			renderData = fiber.Map{
				"headTitle":       "От слона к суслику - статьи про PHP, Go, алгоритмы",
				"headDescription": "список статей",
				"headKeywords":    "Дмитрий Гнатенко, программист, PHP, Go, Golang, программирование, статьи, блог",
				"pageTitle":       "Список статей",
				"articles":        articlesDTO,
			}

			sp.GetCacheService().Set(allPreviewArticlesCacheKey, renderData)
		}

		return ctx.Render("index", renderData, "_layout")
	}
}
